@@ -7,6 +7,10 @@
 #               02/20/04 - P. Harvey Added flag to disable SeekTest in new()
 #               11/18/04 - P. Harvey Fixed bug with seek relative to end of file
 #               01/02/05 - P. Harvey Added DEBUG code
+#               01/09/06 - P. Harvey Fixed bug in ReadLine() when using
+#                          multi-character EOL sequences
+#               02/20/06 - P. Harvey Fixed bug where seek past end of file could
+#                          generate "substr outside string" warning
 #
 # Notes:        Calls the normal file i/o routines unless SeekTest() fails, in
 #               which case the file is buffered in memory to allow random access.
@@ -15,7 +19,7 @@
 #
 #               May also be used for string i/o (just pass a scalar reference)
 #
-# Legal:        Copyright (c) 2004-2005 Phil Harvey (phil at owl.phy.queensu.ca)
+# Legal:        Copyright (c) 2004-2006 Phil Harvey (phil at owl.phy.queensu.ca)
 #               This library is free software; you can redistribute it and/or
 #               modify it under the same terms as Perl itself.
 #------------------------------------------------------------------------------
@@ -27,7 +31,7 @@ require 5.002;
 require Exporter;
 
 use vars qw($VERSION @ISA @EXPORT_OK);
-$VERSION = '1.02';
+$VERSION = '1.04';
 @ISA = qw(Exporter);
 
 # constants
@@ -174,10 +178,13 @@ sub Read($$$)
         }
         # number of bytes left in data buffer
         $num = $self->{LEN} - $self->{POS};
-        if ($len > $num) {
-            $rtnVal = $num;
-        } else {
+        if ($len <= $num) {
             $rtnVal = $len;
+        } elsif ($num <= 0) {
+            $_[0] = '';
+            return 0;
+        } else {
+            $rtnVal = $num;
         }
         # return data from our buffer
         $_[0] = substr(${$self->{BUFF_PT}}, $self->{POS}, $rtnVal);
@@ -218,7 +225,7 @@ sub ReadLine($$)
             }
             # scan and read until we find the EOL (or hit EOF)
             for (;;) {
-                $pos = index(${$self->{BUFF_PT}}, $/, $pos) + 1;
+                $pos = index(${$self->{BUFF_PT}}, $/, $pos) + length($/);
                 last if $pos > 0;
                 $pos = $self->{LEN};    # have scanned to end of buffer
                 $num = read($fp, $buff, $CHUNK_SIZE) or last;
@@ -227,7 +234,7 @@ sub ReadLine($$)
             }
         } else {
             # string i/o
-            $pos = index(${$self->{BUFF_PT}}, $/, $pos) + 1;
+            $pos = index(${$self->{BUFF_PT}}, $/, $pos) + length($/);
             $pos <= 0 and $pos = $self->{LEN};
         }
         # read the line from our buffer
