@@ -6,38 +6,53 @@
 # Revisions:    07/08/05 - P. Harvey Created
 #
 # References:   1) http://partners.adobe.com/public/developer/en/ps/5002.EPSF_Spec.pdf
-#               2) http://partners.adobe.com/public/developer/en/illustrator/sdk/AI7FileFormat.pdf
+#               2) http://partners.adobe.com/public/developer/en/ps/5001.DSC_Spec.pdf
+#               3) http://partners.adobe.com/public/developer/en/illustrator/sdk/AI7FileFormat.pdf
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::PostScript;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.04';
+$VERSION = '1.09';
+
+sub WritePS($$);
+sub CheckPS($$$);
 
 # PostScript tag table
 %Image::ExifTool::PostScript::Main = (
+    WRITE_PROC => \&WritePS,
+    CHECK_PROC => \&CheckPS,
+    PREFERRED => 1, # always add these tags when writing
     GROUPS => { 2 => 'Image' },
-    # Note: Make all of these tags priority 0 since the first one found
-    # at the start of the file should take priority.
-    Author      => { Priority => 0, Groups => { 2 => 'Author' } },
+    # Note: Make all of these tags priority 0 since the first one found at
+    # the start of the file should take priority (in case multiples exist)
+    Author      => { Priority => 0, Groups => { 2 => 'Author' }, Writable => 'string' },
+    BoundingBox => { Priority => 0 },
+    Copyright   => { Priority => 0, Writable => 'string' }, #2
     CreationDate => {
         Name => 'CreateDate',
         Priority => 0,
         Groups => { 2 => 'Time' },
+        Writable => 'string',
     },
-    Creator     => { Priority => 0 },
-    For         => { Priority => 0, Notes => 'found in AI files'},
-    Keywords    => { Priority => 0 },
+    Creator     => { Priority => 0, Writable => 'string' },
+    ImageData   => { Priority => 0 },
+    For         => { Priority => 0, Writable => 'string', Notes => 'for whom the document was prepared'},
+    Keywords    => { Priority => 0, Writable => 'string' },
     ModDate => {
         Name => 'ModifyDate',
         Priority => 0,
         Groups => { 2 => 'Author' },
+        Writable => 'string',
     },
-    Subject     => { Priority => 0 },
-    Title       => { Priority => 0 },
+    Pages       => { Priority => 0 },
+    Routing     => { Priority => 0, Writable => 'string' }, #2
+    Subject     => { Priority => 0, Writable => 'string' },
+    Title       => { Priority => 0, Writable => 'string' },
+    Version     => { Priority => 0, Writable => 'string' }, #2
     # these subdirectories for documentation only
     BeginPhotoshop => {
         Name => 'PhotoshopData',
@@ -59,6 +74,53 @@ $VERSION = '1.04';
     },
 );
 
+# composite tags
+%Image::ExifTool::PostScript::Composite = (
+    GROUPS => { 2 => 'Image' },
+    # BoundingBox is in points, not pixels,
+    # but use it anyway if ImageData is not available
+    ImageWidth => {
+        Desire => {
+            0 => 'PostScript:ImageData',
+            1 => 'PostScript:BoundingBox',
+        },
+        ValueConv => 'Image::ExifTool::PostScript::ImageSize(\@val, 0)',
+    },
+    ImageHeight => {
+        Desire => {
+            0 => 'PostScript:ImageData',
+            1 => 'PostScript:BoundingBox',
+        },
+        ValueConv => 'Image::ExifTool::PostScript::ImageSize(\@val, 1)',
+    },
+);
+
+# add our composite tags
+Image::ExifTool::AddCompositeTags('Image::ExifTool::PostScript::Composite');
+
+#------------------------------------------------------------------------------
+# AutoLoad our writer routines when necessary
+#
+sub AUTOLOAD
+{
+    return Image::ExifTool::DoAutoLoad($AUTOLOAD, @_);
+}
+
+#------------------------------------------------------------------------------
+# Get image width or height
+# Inputs: 0) value list ref (ImageData, BoundingBox), 1) true to get height
+sub ImageSize($$)
+{
+    my ($vals, $getHeight) = @_;
+    my ($w, $h);
+    if ($$vals[0] and $$vals[0] =~ /^(\d+) (\d+)/) {
+        ($w, $h) = ($1, $2);
+    } elsif ($$vals[1] and $$vals[1] =~ /^(\d+) (\d+) (\d+) (\d+)/) {
+        ($w, $h) = ($3 - $1, $4 - $2);
+    }
+    return $getHeight ? $h : $w;
+}
+
 #------------------------------------------------------------------------------
 # Set PostScript format error warning
 # Inputs: 0) ExifTool object reference, 1) error string
@@ -66,6 +128,8 @@ $VERSION = '1.04';
 sub PSErr($$)
 {
     my ($exifTool, $str) = @_;
+    # set file type if not done already
+    $exifTool->SetFileType('PS') unless $exifTool->GetValue('FileType');
     $exifTool->Warn("PostScript format error ($str)");
     return 1;
 }
@@ -108,31 +172,22 @@ sub ProcessPS($$)
 {
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
-    my $data;
+    my ($data, $dos);
 #
-# determine postscript file type
+# determine if this is a postscript file
 #
     $raf->Read($data, 4) == 4 or return 0;
     # accept either ASCII or DOS binary postscript file format
     return 0 unless $data =~ /^(%!PS|\xc5\xd0\xd3\xc6)/;
-    # this appears to be in PostScript format
-    $exifTool->SetFileType('PostScript');
     # process DOS binary file header
     if ($data =~ /^\xc5\xd0\xd3\xc6/) {
-        $raf->Read($data, 26) == 26 or return PSErr($exifTool,'truncated header');
+        # save DOS header then seek ahead and check PS header
+        $raf->Read($dos, 26) == 26 or return 0;
         SetByteOrder('II');
-        # extract information from embedded TIFF file
-        # (set Parent to '' to avoid setting FileType tag again)
-        my %dirInfo = (
-            Parent => '',
-            RAF => $raf,
-            Base => Get32u(\$data, 16),
-        );
-        $exifTool->ProcessTIFF(\%dirInfo);
-        # extract information from PostScript section
-        $raf->Seek(Get32u(\$data, 0), 0);
-        unless ($raf->Read($data, 4) == 4 and $data =~ /^%!/) {
-            return PSErr($exifTool, 'invalid PS header');
+        unless ($raf->Seek(Get32u(\$dos, 0), 0) and
+                $raf->Read($data, 4) == 4 and $data eq '%!PS')
+        {
+            return PSErr($exifTool, 'invalid header');
         }
     }
 #
@@ -140,8 +195,31 @@ sub ProcessPS($$)
 #
     my $oldsep = SetInputRecordSeparator($raf);
     $oldsep or return PSErr($exifTool, 'invalid PS data');
+
+    # set file type (PostScript or EPS)
+    $raf->ReadLine($data) or return 0;
+    $exifTool->SetFileType($data =~ /EPSF/ ? 'EPS' : 'PS');
 #
-# parse the file
+# extract TIFF information from DOS header
+#
+    if ($dos) {
+        my $base = Get32u(\$dos, 16);
+        if ($base) {
+            my $pos = $raf->Tell();
+            # extract information from TIFF in DOS header
+            # (set Parent to '' to avoid setting FileType tag again)
+            my %dirInfo = (
+                Parent => '',
+                RAF => $raf,
+                Base => $base,
+            );
+            $exifTool->ProcessTIFF(\%dirInfo) or $exifTool->Warn('Bad embedded TIFF');
+            # position file pointer to extract PS information
+            $raf->Seek($pos, 0);
+        }
+    }
+#
+# parse the postscript
 #
     my ($buff, $mode, $endToken);
     my $tagTablePtr = GetTagTable('Image::ExifTool::PostScript::Main');
@@ -178,10 +256,11 @@ sub ProcessPS($$)
             undef $endToken;    # no end token (just look for xpacket end)
             # XMP could be contained in a single line (if newlines are different)
             next unless $data =~ m{<\?xpacket end=.w.\?>$/};
-        } elsif ($data =~ /^%%(\w+): (.*)/s and $$tagTablePtr{$1}) {
+        } elsif ($data =~ /^%%?(\w+): ?(.*)/s and $$tagTablePtr{$1}) {
+            my ($tag, $val) = ($1, $2);
+            # only allow 'ImageData' to have single leading '%'
+            next unless $data =~ /^%%/ or $1 eq 'ImageData';
             # extract information from PostScript tags in comments
-            my $tag = $1;
-            my $val = $2;
             chomp $val;
             if ($val =~ s/^\((.*)\)$/$1/) { # remove brackets if necessary
                 $val =~ s/\) \(/, /g;       # convert contained brackets too
@@ -242,8 +321,8 @@ This code reads meta information from EPS (Encapsulated PostScript), PS
 
 Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 REFERENCES
 
@@ -251,6 +330,8 @@ it under the same terms as Perl itself.
 
 =item L<http://partners.adobe.com/public/developer/en/ps/5002.EPSF_Spec.pdf>
 
+=item L<http://partners.adobe.com/public/developer/en/ps/5001.DSC_Spec.pdf>
+
 =item L<http://partners.adobe.com/public/developer/en/illustrator/sdk/AI7FileFormat.pdf>
 
 =back
