@@ -432,8 +432,8 @@ Sigma and Foveon X3F images.
 
 Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 REFERENCES
 
@@ -446,6 +446,7 @@ it under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<Image::ExifTool::TagNames/SigmaRaw Tags>,
+L<Image::ExifTool::Sigma(3pm)|Image::ExifTool::Sigma>,
 L<Image::ExifTool(3pm)|Image::ExifTool>
 
 =cut
